'****************************************************************
'*  Name    : Test_LED_uart1_47k40_HSM.BAS                      *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2017 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 31/10/2017                                        *
'*  Version : 1.0 Bootloader in the High Memory Side            *
'*          : Powered by Proton Basic Compiler V3.6.0.3         *
'*          : of Crownhill Associates Ltd                      *
'*  Notes   : Using the Debounce library: DebounceInt.Inc       *
'*          : with the Amicus18 board.                          *
'*          : This special code is used to check the bootloader *
'****************************************************************

'===============================================================================
(*
    NVMCON1
    The NVMREG<1:0> control bits determine If the Access will be To Data EEPROM
    Memory locations. PFM locations Or User IDs, Configuration bits, Rev ID And Device ID.
    When NVMREG<1:0> = 00, any subsequent operations will operate On the Data EEPROM Memory.
    When NVMREG<1:0> = 10, any subsequent operations will operate On the program memory.
    When NVMREG<1:0> = x1, any subsequent operations will operate On the
    Configuration bits, User IDs, Rev ID And Device ID.
*)
'===============================================================================
    ' Xtal avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10
    ' The USER must copy this "Declare" in the user main project.
    Device = 18F47K40
    Declare Xtal = 64

    ' DEFINE THE PLL: On / OFF
    $define PLL_ConfigFuses On

    ' Enable the to use the internal Oscillator: Xtal avaibled: 64, 48, 32, 16, 12
    ' Uncomment the next line to enable the configuration.
    '$define _InternalOSC_ 

    ' Enable the Line to use the CLOCK OUT FUNCTION: 
    ' If CLKOUT function is enabled, CLKOUT on RA6 & Port function On RA7 otherwize Port function On RA6 & RA7.     
    '$define _CLKOUT_Function_

'===============================================================================
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = On
    Declare Bootloader = Off
    Declare Watchdog = On
    Declare Eeprom_Address = $310000
    Declare Access_Upper_64K = On
    Symbol NbBytesIDlocs = 8
'===============================================================================
' Define the PORT & PIN of the test LED.
'
    $define _PORT PORTC
    $define _PIN 5
'===============================================================================
' Declare the UART Baud Rate manually: enable ONE line ONLY.

'    $define _BaudRate 9600

'    $define _BaudRate 19200
    
'    $define _BaudRate 38400
    
'    $define _BaudRate 57600
        
'    $define _BaudRate 115200

'-------------------------------------------------------------------------------
' The Declare UART Baudrate is automatic depending of the Xtal to be syncronized with the bootloader.
    $ifndef _BaudRate
        $if _xtal >= 40
            $ifdef _InternalOSC_
            $define _BaudRate 57600
            $else
            $define _BaudRate 115200
            $endif
        $endif
        $if (_xtal >= 16) And (_xtal < 40)
            $define _BaudRate 57600
        $endif
        $if _xtal < 16
            $define _BaudRate 38400
        $endif
    $endif
'===============================================================================
' Same Baudrate as the Bootloader to check the Software.
'-------------------------------------------------------------------------------
$if _BaudRate = 9600
    Declare Hserial_Baud = 9600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 19200
    Declare Hserial_Baud = 19200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 38400
    Declare Hserial_Baud = 38400
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 57600
    Declare Hserial_Baud = 57600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode  4024
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 115200
    Declare Hserial_Baud = 115200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'===============================================================================
    Symbol CTL_ROM_SIZE = _code
    Symbol CTL_RAM_SIZE = _ram    
'===============================================================================
' Same Config Fuses as Bootloader:
'
Config_Start
$ifdef _InternalOSC_
    FEXTOSC = OFF	            ; External Oscillator disabled
    RSTOSC = HFINTOSC_64MHZ	    ; HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
    $ifdef CLKOUT_Function
    CLKOUTEN = On	        ;CLKOUT: FOSC/4, function is enabled, Port function on RA6 is disabled. 
    $else
    CLKOUTEN = OFF	        ;CLKOUT function is disabled, Port function on RA6 is enabled. 
    $endif    
$else
    $if PLL_ConfigFuses = On
        $if (_xtal >= 10) And (_xtal <= 64)
        FEXTOSC = HS	        ; HS (crystal oscillator) above 8 MHz; PFM set to high power
        RSTOSC = EXTOSC_4PLL	; EXTOSC with 4x PLL, with EXTOSC operating per FEXTOSC bits
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
    $if PLL_ConfigFuses = OFF
        $if (_xtal >= 10) And (_xtal <= 25)
        FEXTOSC = HS	    ; HS (crystal oscillator) above 8 MHz; PFM set to high power
        RSTOSC = EXTOSC     ; EXTOSC operating per FEXTOSC Bits (device manufacturing default)
        $else
            $if (_xtal > 25) And (_xtal <= 64)        
            FEXTOSC = ECH	    ; ECH  External Clock High-Power mode (above 8 MHz)
            RSTOSC = EXTOSC     ; EXTOSC operating per FEXTOSC Bits (device manufacturing default)        
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
        $endif
        $ifdef CLKOUT_Function
        CLKOUTEN = On	        ;CLKOUT: FOSC/4, function is enabled, Port function on RA6 is disabled. 
        $else
        CLKOUTEN = OFF	        ;CLKOUT function is disabled, Port function on RA6 is enabled. 
        $endif  
    $endif
$endif
    CSWEN = On	            ;Writing to NOSC and NDIV is allowed
    FCMEN = OFF	    ;Fail-Safe Clock Monitor disabled
    MCLRE = EXTMCLR	;If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
    PWRTE = On	     ;Power up timer enabled
    LPBOREN = On	;ULPBOR enabled
    BOREN = On	     ;Brown-out Reset enabled according to SBOREN
    BORV = VBOR_2P45	;Brown-out Reset Voltage (VBOR) set to 2.45V
    ZCD = OFF	      ;ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
    PPS1WAY = OFF	;PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
    STVREN = On	    ;Stack full/underflow will cause Reset
    Debug = OFF	    ;Background debugger disabled
    XINST = OFF	    ;Extended Instruction Set and Indexed Addressing Mode disabled
    WDTCPS = WDTCPS_5	;Divider ratio 1:1024
    WDTE = SWDTEN	;WDT enabled/disabled by SWDTEN bit
    WDTCWS = WDTCWS_7	;window always open (100%); software control; keyed access not required
    WDTCCS = SC	    ;Software Control
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000-00FFFFh) not write-protected
    WRT4 = OFF	;Block 4 (010000-013FFFh) not write-protected
    WRT5 = OFF	;Block 5 (014000-017FFFh) not write-protected
    WRT6 = OFF	;Block 6 (018000-01BFFFh) not write-protected
    WRT7 = OFF	;Block 7 (01C000-01FFFFh) not write-protected
    WRTC = On	;Configuration registers (300000-30000Bh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    SCANE = On	;Scanner module is available for use, SCANMD bit can control the module
    LVP = On	;Low voltage programming enabled. MCLR/VPP pin function is MCLR. MCLRE configuration bit is ignored
    Cp = OFF	;UserNVM code protection disabled
    CPD = OFF	;DataNVM code protection disabled
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTR4 = OFF	;Block 4 (010000-013FFFh) not protected from table reads executed in other blocks
    EBTR5 = OFF	;Block 5 (014000-017FFFh) not protected from table reads executed in other blocks
    EBTR6 = OFF	;Block 6 (018000-01BFFFh) not protected from table reads executed in other blocks
    EBTR7 = OFF	;Block 7 (01C000-01FFFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
'###############################################################################
' THE USER SOFTWARE STARTS:
'###############################################################################
'===============================================================================
    Include "D:\MyIncFiles\Devicelist.inc"
'===============================================================================
    Include "D:\MyIncFiles\SYS_18F27K40_47K40.INC"
'===============================================================================
    ' PPS library to declare the pins of the PIC.
    Include "D:\MyIncFiles\PPS_18F27_47K40.INC"
'===============================================================================
    ' Library to disable/enable the interrupts.
    Include "D:\MyIncFiles\GIEDisable.Inc"
'===============================================================================
    ' Library for HRSOut.
    $define _2StopBits_
    Include "D:\MyIncFiles\HRsout_K40.inc"
'===============================================================================
    $define ERASE_FLASH_BLOCKSIZE 128        ' For the PIC18F27K40.
'    Include "D:\MyIncFiles\ORG_SyncBlock.Inc"
'===============================================================================
' New Definitions of the library flags.
'    Dim F_TMR2IF As PIR4Bits_TMR2IF     ' TMR2 To PR2 Match Interrupt Flag Bit
'    Dim F_TMR2IE As PIE4Bits_TMR2IE     ' TMR2 To PR2 Match Interrupt Enable bi
'    Dim F_TMR2ON As T2CONBits_TMR2ON    ' Timer2 On Bit 
'    Dim F_IPEN As INTCONBits_IPEN         ' Disable priority levels On interrupts (PIC16CXXX Compatibility mode)
'    Dim F_TMR2IP As IPR4Bits_TMR2IP     ' TMR2 to PR2 Match Interrupt Priority bit
'------------------------------------------------------------------------------- 
'ASCII characters  
'    Symbol Home = 1
'    Symbol Bell = 7
'    Symbol BkSp = 8
'    Symbol Tab = 9
'    Symbol CR = 13
'    Symbol LF = $0A     ' Line Feed Char    
'    Symbol True = 1     ' Boolean True
'    Symbol False = 0    ' Boolean False
'    Symbol Null = 0     ' Null character 
'    Symbol ACK = 6
'    Symbol STX = 2
'    Symbol ETX = 3
'    Symbol SPACE = $20
'    Symbol DEL = 127    
'    Symbol ESC = $1B    ' Escape
'    Symbol BS = 08      ' Backspace 
'===============================================================================
    Dim NbIDlocs As Byte
    $define Read_IDLOCS(pNumbers) '
    NbIDlocs = pNumbers           '
    GoSub Read_IDLOCS_Sub

'-------------------------------------------------------------------------------
' Declare some variables.
    Dim EEPADDR As NVMADRL.Word     ' NVMADRL and NVMADRH
    Dim LoopByte As Byte
    Dim Temp1 As Byte
    Dim Temp2 As Byte
    Dim Temp3 As Byte
    Dim Temp4 As Byte        
    Dim DataWord As Word
    Dim EepAddress As Word
    Dim PICPartNumber As Word
    Dim PICRevision As Word
    Dim CounterEep As Byte
    Dim RevASCII As Byte
    Dim Address1 As Word
    Dim Address2 As Word       
    Dim Address3 As Word
    Dim Address4 As Word
    Dim YesASCII As Byte
    Dim IDLOCS_String As String * 16        
        
    Dim CodeTestFlags As Byte
    Dim F_LED8_On As CodeTestFlags.0
    Dim F_LED8_Off As CodeTestFlags.1    
    Dim F_LED9_On As CodeTestFlags.2
    Dim F_LED9_Off As CodeTestFlags.3
'===============================================================================
' Running with Internal oscillator ONLY.
' CONFIG FUSES:
' FOSC = INTIO67    ;Internal oscillator block, port function on RA6 and RA7
' HFOFST = OFF	    ;HFINTOSC output and ready status are delayed by the oscillator stable status
' Time for HF-INTOSC to Stabilize: 1mS, Time For PLL to Stabilize: 2mS
' Delay for code: 25mS (more or less)
'*******************************************************************************
'* To save code it is possible to disable theses lines of code,                *
'* because the Internal Oscillator is configurated by the Bootloader           *
'* or to change the HFINTOSC frequency                                         *
'* FOSC avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10        *
'*******************************************************************************
' Perform the locking sequence for PPS assignment
' Input     : None
' Output    : None
' Clear PPSLOCKED bit to enable writes
'
$ifndef PPS_EnableWrite
$define PPS_EnableWrite() '
    GIE_Disable()         '
    NVMCON2 = $55         '
    NVMCON2 = $AA         '
    PPSLOCK.0 = 0         '
    GIE_Restore()
$endif

'-------------------------------------------------------------------------------
' Perform the locking sequence for PPS assignment
' Input     : None
' Output    : None
' Set PPSLOCKED bit to disable writes
'
$ifndef PPS_DisableWrite
$define PPS_DisableWrite() '
    GIE_Disable()          '
    NVMCON2 = $55          '
    NVMCON2 = $AA          '
    PPSLOCK.0 = 1          '
    GIE_Restore()
$endif
    
'===============================================================================
' Watchdog configuration.
    WDTCON1 = %00000111         ' Window = 100%, WATCHDOG TIMER CONTROL REGISTER 1
'===============================================================================
$ifdef _InternalOSC_
INT_OSC:
' Write the number of bytes if some code is written.
' See the Datasheet to configure the internal oscillator.
    OSCCON1 = %01100000                 ' HFINTOSC is used for 64Mhz as start up.

    ' OSCFRQ: HFINTOSC FREQUENCY SELECTION REGISTER
    $if _xtal = 64
    OSCFRQ = %00001000
    $else
        $if _xtal = 48
        OSCFRQ = %00000111    
        $else    
            $if _xtal = 32
            OSCFRQ = %00000110    
            $else   
                $if _xtal = 16
                OSCFRQ = %00000101    
                $else        
                    $if _xtal = 12
                    OSCFRQ = %00000100    
                    $else  
                        $error "Error in (Declare Xtal) with Internal Oscillator"
                    $endif
                $endif                    
            $endif
        $endif                        
    $endif
    While OSCSTATBits_HFOR = 0 : Wend   ' HFINTOSC Oscillator Ready bit. 1 = The oscillator is ready to be used
$endif
    DelayMS 25
'===============================================================================
'*******************************************************************************
'* Name    : On_Low_Interrupt                                                  *
'* Purpose : To test the ISR_Low Jump by the Bootloader.                       *
'*         : Clock = FOSC/4 => 64Mhz/4 = 16Mhz                                 *
'*******************************************************************************
    On_Low_Interrupt GoTo ISR_Low
    GoTo OverLowInterrupt
'===============================================================================   
ISR_Low:
    ' Timer is used to Toggle the LED9 every second.
    Context Save
    If PIR0Bits_TMR0IF = 1 Then   ' Check the Timer0 Interrupt flag
        PIR0bits_TMR0IF = 0
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Toggle _PORT._PIN                 ' LED = One second ON, one second OFF.
    EndIf
    Context Restore
'===============================================================================
OverLowInterrupt:    
'===============================================================================
MAIN_TEST:
    GIE_OFF     ' Disable all interrupts
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    DelayMS 50
'-------------------------------------------------------------------------------    
    PIR2 = 0
    PIE2 = 0
    IPR2 = 0                    
'
    T4CON = 0
    T6CON = 0
    PWM3CON = 0
    PWM4CON = 0    
' Declare all digital   
    ADCON0 = 0
    ADCON1 = 15
    $ifdef _ADCON2
    ADCON2 = 0    
    $endif
    $ifdef _CM1CON0
    CM1CON0 = 0
    $endif
    $ifdef _CM2CON0
    CM2CON0 = 0
    $endif
    $ifdef _CM2CON1
    CM2CON1 = 0
    $endif
    $ifdef _ANSELA
    ANSELA = 0
    $endif
    $ifdef _ANSELB
    ANSELB = 0
    $endif
    $ifdef _ANSELC
    ANSELC = 0
    $endif
    $ifdef _ANSELD
    ANSELD = 0
    $endif    
' The eeprom is not used for the moment.
    NVMCON1 = 0

TRISPORT:
' PORTS
    ' TRIS the LED pin
    Output _PORT._PIN
    ' Clear all PORTS
    LATA = 0
    LATB = 0
    LATC = 0
    DelayMS 50
'-------------------------------------------------------------------------------
UART_CONFIG:
' Config the UART pins to digital.
    GIE_Disable()               ' Disable all interrupts.
    ' The interrupts are disabled.
    PPSLOCK = $55
    PPSLOCK = $AA
    PPSLOCK.0 = 0   ' Unlock PPS.
    RX1PPS = 23     ' Declare Pin RC7 input
    RC6PPS = 9      ' Declare Pin RC6 output
    PPSLOCK = $55
    PPSLOCK = $AA
    PPSLOCK.0 = 1   ' Lock PPS.
    GIE_Restore()   ' Enable all interrupts.
'-------------------------------------------------------------------------------
' Config the UART pins to digital with the library.
'    PPS_EnableWrite()
'    iPPSInput(cIn_Fn_RX1DT1,cIn_Pin_RC7)       ' RX1 <= RC7 input 
'    iPPSOutput(cOut_Pin_RC6,cOut_Fn_TX1CK1)    ' RC6 output <= TX1
'    PPS_DisableWrite()
'-------------------------------------------------------------------------------
    ANSELC.7 = 0    ' Pin RX1 to digital
    ANSELC.6 = 0    ' Pin TX1 to digital
    Input PORTC.7   ' Declare Pin RX1 for Input.
    Output PORTC.6  ' Declare Pin TX1 for output.        
    ODCONC.7 = 0    ' ODCONx: OPEN-DRAIN CONTROL REGISTER, 0 = Output drives both high-going and low-going signals (source and sink current).
    SLRCONC = 0     ' SLRCONx: SLEW RATE CONTROL REGISTER, 0 = Port pin slews at maximum rate. 
    '
    ' Clean the UART
    CleanUART1()
    DelayMS 50
'*******************************************************************************
' Write some String in the Data eeprom memory.
    '---------------------------------------------------------------------------
E2p_AddressString1 EData "Testing PIC18F47K40 AGV Bootloader V5.0 written in High Side of ROM.",0

E2p_AddressString2 EData "New Universal PIC18 Bootloader by Alberto Freixanet. ",0

E2p_AddressString3 EData "A 8 digits Password could be needed to download the user code. ",0

E2p_AddressString4 EData As Byte 10,10,10,10

'*******************************************************************************
STARTING_TEST:
'===============================================================================
' The Configuration of the Timer0 is different.
' Configuration of the Timer0 to toggle a LED every 1.048Sec 
' Xtal avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10
    ' Timer0 is configured As a 16-Bit timer/Counter, 1:256 prescale value: 1.048Sec 
    $if _xtal = 64
    $define TMR0_PreScaler 8
    $define TMR0_PostScaler 0
    $endif
    $if _xtal = 48
    $define TMR0_PreScaler 6
    $define TMR0_PostScaler 2
    $endif
    $if _xtal = 32
    $define TMR0_PreScaler 7
    $define TMR0_PostScaler 0
    $endif
    $if _xtal = 16
    $define TMR0_PreScaler 6
    $define TMR0_PostScaler 0
    $endif
    $if _xtal = 12
    $define TMR0_PreScaler 6
    $define TMR0_PostScaler 2
    $endif    
    '---------------------------------------------------------------------------
    GIE_Disable()               ' Disable all interrupts.
    WPUB = 0                    ' All PORTB pull-ups are disabled.
    INTCONBits_IPEN = 1         ' Enable priority levels on interrupts.
    T0CON0 = %00010000 | TMR0_PostScaler   ' 16 Bits Timer, Postscaler, Timer0 = OFF
    T0CON1 = %01000000 | TMR0_PreScaler    ' FOSC/4, Prescaler
    '---------------------------------------------------------------------------
    IPR0Bits_TMR0IP = 0         ' Timer0 for Low Interrupt
    PIR0Bits_TMR0IF = 0         ' Clear the Timer0 interrupt flag to begin.
    PIE0Bits_TMR0IE = 1         ' Enable the Timer0 interrupts.
    INTCON.6 = 1                ' Enables all low priority interrupts
    TMR0H = 0                   ' Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CON0Bits_T0EN = 1         ' Start the Timer0.
    GIE_Restore()
'===============================================================================
    HRSOut CR,LF
'-------------------------------------------------------------------------------    
    HRSOut "USER CODE STARTS!",CR,LF,CR,LF 
    HRSOut "TESTING THE HRSOut COMMAND!",CR,LF,CR,LF 
'===============================================================================
READ_CONFIG:
' Read config fuses in memory map.
    Set_Bank TBLPTRL
    Asm
    Movlw Low (Config2L)
    Movwf TBLPTRL
    Movlw High (Config2L)
    Movwf TBLPTRH
    Movlw upper (Config2L)
    Movwf TBLPTRU
    EndAsm
    GIE_Disable()                   ' PIC errata, DS80000713A.
    NVMCON1 = %11000000             ' Access Configuration registers
    Tblrd*+                         ' Read one memory position.  
    Temp3 = TABLAT                  ' 300002h Config2L BOREN1 BOREN0 LPBOREN    PWRTE MCLRE
    Tblrd*+                         ' Read one memory position.  
    Temp2 = TABLAT                  ' 300003h Config2H XINST  Debug STVREN PPS1WAY ZCD BORV1 BORV0
    Tblrd*+                         ' Read one memory position.  
    Temp1 = TABLAT                  ' 300004h CONFIG3L  WDTE<1:0> WDTCPS<4:0>
    NVMCON1 = 0                     ' Disable Access the Configuration registers
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0                     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
'-------------------------------------------------------------------------------
START_CONFIG:
    TBLPTRU = 0
    HRSOut "CONFIG FUSES: ",CR,LF 
'
    HRSOut "Configuration Word 2L: %", Bin8 Temp3,CR,LF
    Temp3 = (Temp3 >> 6) & 3
    If Temp3.5 = 0 Then
        HRSOut "Low-Power Brown-out Reset is enabled!",CR,LF
        If Temp3 = 3 Then
            HRSOut "Brown-out Reset enabled, SBOREN bit is ignored!"    
        ElseIf Temp3 = 2 Then
            HRSOut "Brown-out Reset enabled while running, disabled in Sleep; SBOREN is ignored!"   
        ElseIf Temp3 = 1 Then
            HRSOut "Brown-out Reset enabled according to SBOREN!"
        ElseIf Temp3 = 0 Then
            HRSOut "Brown-out Reset disabled!"
        EndIf
    Else
        HRSOut "Low-Power Brown-out Reset is disabled!"
    EndIf
    HRSOut CR,LF
    If BORCONBits_SBOREN = 1 Then
        HRSOut "Brown-out Reset enabled by SBOREN!" 
    Else
        HRSOut "Brown-out Reset disabled by SBOREN!"     
    EndIf  
    HRSOut CR,LF,CR,LF
'
    HRSOut "Configuration Word 2H: %", Bin8 Temp2,CR,LF
    If Temp2.3 = 1 Then    
        HRSOut "The PPSLOCKED bit can only be set once after an unlocking sequence is executed"
    Else
        HRSOut "The PPSLOCKED bit can be set and cleared as needed"    
    EndIf
    HRSOut CR,LF,CR,LF
'
    HRSOut "Configuration Word 3L: %", Bin8 Temp1,CR,LF
    Temp1 = (Temp1 >> 5) & 3
    If Temp1 = 3 Then
        HRSOut "WDT enabled regardless of Sleep; SEN bit in WDTCON0 is ignored!"    
    ElseIf Temp1 = 2 Then
        HRSOut "WDT enabled While Sleep = 0; Suspended when Sleep = 1; SEN bit in WDTCON0 is ignored!"   
    ElseIf Temp1 = 1 Then
        HRSOut "WDT enabled/disabled by SEN Bit in WDTCON0!",CR,LF
        HRSOut "Be carrefull the WDT could be active!" 
    ElseIf Temp1 = 0 Then
        HRSOut "WDT disabled, SEN Bit in WDTCON0 is ignored!"
    EndIf
    HRSOut CR,LF,CR,LF
'===============================================================================
' Check the SWDTEN Bit.
    If Temp1 = 1 Then
        HRSOut "After Reset the "
        If WDTCON0Bits_SWDTEN = 1 Then
            HRSOut "SWDTEN Bit = 1 (error)",CR,LF
        Else
            HRSOut "SWDTEN Bit = 0 (good, by code)",CR,LF
        EndIf
        HRSOut "Now Set the SWDTEN Bit",CR,LF
        WDTCON0Bits_SWDTEN = 1           ' Enable the Watchdog
        If WDTCON0Bits_SWDTEN = 1 Then
            HRSOut "The SWDTEN Bit is set => Watchdog Enabled.",CR,LF
        Else
            HRSOut "The SWDTEN Bit is Cleared => Watchdog Disabled.",CR,LF    
        EndIf
    EndIf
    HRSOut CR,LF
'===============================================================================
    $ifdef _InternalOSC_
    HRSOut "HFINTOSC IS IN USE.",CR,LF 
    $else
    HRSOut "EXTOSC IS IN USE.",CR,LF     
    $endif
    HRSOut "OSCCON1: OSCILLATOR CONTROL REGISTER 1: ", IBin8 OSCCON1,CR,LF          
    HRSOut "OSCCON2: OSCILLATOR CONTROL REGISTER 2: ", IBin8 OSCCON2,CR,LF
    HRSOut "OSCCON3: OSCILLATOR CONTROL REGISTER 3: ", IBin8 OSCCON3,CR,LF
    HRSOut "OSCSTAT: OSCILLATOR STATUS  REGISTER 1: ", IBin8 OSCSTAT,CR,LF
    $ifdef _InternalOSC_
    HRSOut "OSCFRQ:  HFINTOSC FREQUENCY SELECTION REGISTER: ", IBin8 OSCFRQ,CR,LF    
    HRSOut "OSCTUNE: HFINTOSC TUNING REGISTER: ", IBin8 OSCTUNE,CR,LF     
    $endif
    HRSOut CR,LF
'===============================================================================
' Read the PIC ID from Config fuse.
    Set_Bank TBLPTRL
    Asm
    Movlw Low (DEVID1-2)
    Movwf TBLPTRL
    Movlw High (DEVID1-2)
    Movwf TBLPTRH
    Movlw upper (DEVID1-2)
    Movwf TBLPTRU
    EndAsm
    GIE_Disable()                  ' PIC errata, DS80000713A.
    NVMCON1 = %11000000             ' Access Configuration registers 
    ' Read Revision
    Tblrd*+
    PICRevision.LowByte = TABLAT
    Tblrd*+
    PICRevision.HighByte = TABLAT     
    ' Read Part Number
    Tblrd*+
    PICPartNumber.LowByte = TABLAT
    Tblrd*+
    PICPartNumber.HighByte = TABLAT
    NVMCON1 = 0                     ' Disable Access the Configuration registers
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0                     ' Cleared for security
    '---------------------------------------------------------------------------
    ' Calculate the PIC Part Number & revision
    ' ASCII calculation
    PICRevision.HighByte = (PICRevision.HighByte & %00001111) << 2
    Temp4 = PICRevision.LowByte >> 6
    RevASCII = PICRevision.HighByte | Temp4
    RevASCII = RevASCII + "A"
    ' Number calculation
    PICRevision.LowByte = PICRevision.LowByte & %00111111
'-------------------------------------------------------------------------------
    ' Print the Device Name to the Terminal.
    $ifndef __DEVICE_ID
        $error "The Devicelist.inc is missing!!"
    $endif
    Select PICPartNumber
        Case __DEVICE_ID
            HRSOut "Device = ", __DEVICE_NAME, CR,LF
        Case Else
            HRSOut CR,LF    
            HRSOut "The PIC Device number read from the Board is not correct!",CR,LF   
            HRSOut CR,LF        
    EndSelect        
'-------------------------------------------------------------------------------
    HRSOut "PIC Part Number: ", Dec PICPartNumber, "/$", Hex4 PICPartNumber 
    HRSOut "; Revision: ", RevASCII, Dec3 PICRevision.LowByte, "/", RevASCII,"$", Hex2 PICRevision.LowByte,CR,LF
    HRSOut "FOSC: ", Dec _xtal," Mhz;"
    $ifdef _InternalOSC_
    HRSOut " Internal Oscillator ",CR,LF    
    $else
    HRSOut " Xtal Oscillator ",CR,LF    
    $endif
    HRSOut "WRITE BLOCK: ", Dec _block," Bytes",CR,LF 
    HRSOut "ERASE BLOCK: ", Dec ERASE_FLASH_BLOCKSIZE," Bytes",CR,LF
    HRSOut "EEPROM MEMORY: ", Dec _eeprom," Bytes",CR,LF
    HRSOut "ROM MEMORY: ", Dec _code," Bytes",CR,LF     
    HRSOut "RAM MEMORY: ", Dec _ram," Bytes",CR,LF
    HRSOut "BANKA END: ", Dec _bankAend,CR,LF
    HRSOut "RAM BANKS: ", Dec _rambanks,CR,LF
    HRSOut "Number UART: 1 of ", Dec _uart,CR,LF  
    HRSOut "UART Baud Rate: ", Dec _BaudRate,CR,LF
'-------------------------------------------------------------------------------
    HRSOut CR,LF
'===============================================================================   
IDLOCS:
' READING THE IDLOCS...(16 x Words)
    ' Reading the ID Locs written in ASM.
    HRSOut "Reading the ID LOCS HEX format: (16 Bytes)",CR,LF
    Read_IDLOCS(16)
    HRSOut "Reading the ID LOCS: (8 Bytes)",CR,LF
    Read_IDLOCS(8)
    HRSOut CR,LF

    ' Writing new 16 ID Locs.
    #ifdef watchdog_req
    Clrwdt
    #endif
    HRSOut "Writing new ID LOCS: (8 Bytes)",CR,LF
    HRSOut "$F0 $F1 $F2 $F3 $F4 $F5 $F6 $F7",CR,LF 

    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    GIE_Disable()                   ' PIC errata, DS80000713A.
    ' Erase the ID LOCS
    NVMCON1 = %11010100             ' Flash acces + Start erase flash + Setup write flash 
    GoSub WriteSequence_Sub
    ' Write to holding register
    NVMCON1 = %11000000             ' Access Configuration registers
    TABLAT = $F0
    For LoopByte = 0 To NbIDlocs - 1
        Asm
        tblwt*+
        EndAsm
        Inc TABLAT
    Next  
    'Write new ID Locs
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    EndAsm
    NVMCON1 = %11000100            ' Setup write flash
    GoSub WriteSequence_Sub
    GIE_Restore()   ' PIC errata, DS80000713A.
    TBLPTRU = 0     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                    ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
'-------------------------------------------------------------------------------
' READING THE IDLOCS...(16 x Words)
    ' Reading the ID Locs written by code.
    HRSOut "Reading the ID LOCS: (8 Bytes)",CR,LF
    Read_IDLOCS(8)
'-------------------------------------------------------------------------------
' Erase the ID LOCS for next time
    HRSOut "Erasing and Checking the ID LOCS. (16 Bytes)",CR,LF
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    GIE_Disable()                   ' PIC errata, DS80000713A.
    NVMCON1 = %11010100             ' Flash acces + Start erase flash + Setup write flash 
    GoSub WriteSequence_Sub    
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0    
    ' Reading the ID LoCs again.
    Read_IDLOCS(16)
'===============================================================================
STRING_EEP:
    ' Send a String to the terminal read from the data eeprom memory.
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "START EEPROM Strings:",CR,LF
    EepAddress = E2p_AddressString1
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString2
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString3
    GoSub ReadEEPString_Sub
    HRSOut "END EEPROM Strings:",CR,LF,CR,LF
'-------------------------------------------------------------------------------
STRING_ROM:
    HRSOut "START Checking Cread command.",CR,LF
    EepAddress = CdataAddress
    GoSub ReadROMString_Sub
    HRSOut "END Checking CRead command.",CR,LF,CR,LF
'-------------------------------------------------------------------------------
E2P:
    HRSOut "Checking Eread/Ewrite commands.",CR,LF 
    Address1 = E2p_AddressString4
    Address2 = E2p_AddressString4 + 1
    Address3 = E2p_AddressString4 + 2
    Address4 = E2p_AddressString4 + 3   
    HRSOut "Ewrite numbers: 50,125,100,200",CR,LF 
    EWrite  Address1,[50,125,100,200]
    Temp1 = ERead Address1
    Temp2 = ERead Address2
    Temp3 = ERead Address3
    Temp4 = ERead Address4    
    HRSOut "Eread  numbers: ", Dec Temp1,",", Dec Temp2,",", Dec Temp3,",", Dec Temp4,CR,LF 
    HRSOut CR,LF
'-------------------------------------------------------------------------------
    #ifdef __LOW_INTERRUPTS_ENABLED
    HRSOut "Low Interrupts are used!",CR,LF
    #endif
    #ifdef __HIGH_INTERRUPTS_ENABLED
    HRSOut "High Interrupts are used!",CR,LF
    #endif
    HRSOut CR,LF 
'-------------------------------------------------------------------------------
    HRSOut "The LED must toggle now!",CR,LF
    HRSOut CR,LF        
'===============================================================================
MAIN_CODE:
    GIE_ON      ' Enable all interrupts
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        DelayMS 10
    Wend
'===============================================================================
'===============================================================================
'SUBROUTINES:
'===============================================================================
' Read a string written in the eeprom memory.
ReadEEPString_Sub:
    LoopByte = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = ERead EepAddress + LoopByte
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            Inc CounterEep
            If CounterEep > 10 Then     ' Too much addresses erased.
                HRSOut "This String is erased!",CR,LF
                Return     
            EndIf
        Else
            HRSOut Temp1
        EndIf
        Inc LoopByte
    Wend 
'===============================================================================
' Read a string written in the ROM memory.
ReadROMString_Sub:
    LoopByte = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = CRead EepAddress + LoopByte
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            HRSOut "This String is erased!",CR,LF
            Return     
        Else
            HRSOut Temp1
        EndIf
        Inc LoopByte
    Wend     
'===============================================================================
Read_IDLOCS_Sub:
' READING THE IDLOCS...(8 x Words)
'    HRSOut "Reading the ID LOCS: (", Dec NbBytesIDlocs, " Bytes)",CR,LF
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    GIE_Disable()                       ' PIC errata, DS80000713A.
    NVMCON1 = %11000000                 ' Access Configuration registers
    YesASCII = 0
    For LoopByte = 0 To NbIDlocs - 1        ' Start the LoopByte
        Tblrd*+                             ' Read one memory position.
        IDLOCS_String[LoopByte] = TABLAT    ' Load results in ASCII format.
        If TABLAT > 32 And TABLAT < 127 Then 
            Inc YesASCII
        EndIf
    Next      
    NVMCON1 = 0     ' Disable read the Configuration registers
    GIE_Restore()   ' PIC errata, DS80000713A.
    TBLPTRU = 0     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                    ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.

    If YesASCII = NbIDlocs Then
        If NbIDlocs = 8 Then
            IDLOCS_String#8 = 0     ' Last value for the String.
        Else
            IDLOCS_String#16 = 0
        EndIf
        For LoopByte = 0 To NbIDlocs - 1
            Temp1 = IDLOCS_String[LoopByte]
            HRSOut Temp1, " "               ' YES, Print a String.    
        Next
    Else                                    ' NO, Print the Numbers.
        For LoopByte = 0 To NbIDlocs - 1    ' Start the LoopByte
            Temp1 = IDLOCS_String[LoopByte]
            HRSOut IHex2 Temp1, " "
        Next    
    EndIf
    HRSOut CR,LF
    Return
'===============================================================================
WriteSequence_Sub:
    NVMCON2 = $55                    ' EECON2 <- $55
    NVMCON2 = $AA                    ' EECON2 <- $AA
    Set NVMCON1Bits_WR               ' Set WR Bit For write
    Nop
    While NVMCON1Bits_WR = 1 : Wend  ' Waiting to complete.
    NVMCON1 = 0                      ' Disable write
    Return
'===============================================================================
CdataAddress:   CData "HELLO WORLD",0
'===============================================================================
    End
'===============================================================================
' Write some static data to the ID Locations.
' Problem writing ID LOCS in static mode.
'Asm
'__IDLOCS   __IDLOC0,0x77
'__IDLOCS   __IDLOC1,0x76
'__IDLOCS   __IDLOC2,0x75
'__IDLOCS   __IDLOC3,0x74
'__IDLOCS   __IDLOC4,0x73
'__IDLOCS   __IDLOC5,0x72
'__IDLOCS   __IDLOC6,0x71
'__IDLOCS   __IDLOC7,0x70
'EndAsm
Asm
__IDLOCS   IDLOC0,"L"
__IDLOCS   IDLOC1,"O"
__IDLOCS   IDLOC2,"A"
__IDLOCS   IDLOC3,"D"
__IDLOCS   IDLOC4,"E"
__IDLOCS   IDLOC5,"R"
__IDLOCS   IDLOC6,"5"
__IDLOCS   IDLOC7,"0"
EndAsm
'===============================================================================
'===============================================================================


