'****************************************************************
'*  Name    : CodeTest_uart1_27k40_LSM.BAS                      *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2017 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 31/01/2017                                        *
'*  Version : 1.0 Bootloader in the Low Memory Side             *
'*          : Powered by Proton Basic Compiler V3.5.9.0         *
'*          : of Crownhill Associates Ltd                      *
'*  Notes   : Using the Debounce library: DebounceInt.Inc       *
'*          : with the Amicus18 board.                          *
'*  Code powered by the Proton Development Suite Version 3586   *
'*          : This special code is used to check the bootloader *
'****************************************************************


'===============================================================================
(*
    NVMCON1
    The NVMREG<1:0> control bits determine If the Access will be To Data EEPROM
    Memory locations. PFM locations Or User IDs, Configuration bits, Rev ID And Device ID.
    When NVMREG<1:0> = 00, any subsequent operations will operate On the Data EEPROM Memory.
    When NVMREG<1:0> = 10, any subsequent operations will operate On the program memory.
    When NVMREG<1:0> = x1, any subsequent operations will operate On the
    Configuration bits, User IDs, Rev ID And Device ID.
*)
'===============================================================================
' Same device as the Bootloader.

    Device = 18F27K40

'    $define _InternalOSC_
'-------------------------------------------------------------------------------
' COPY the Declares of the bootloader:
' Xtal avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14, 12, 10
    Declare Xtal = 64
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = On
    Declare Bootloader = Off
    Declare Watchdog = On
    Declare Eeprom_Address = $310000
    Declare Access_Upper_64K = On
    Declare PROTON_START_ADDRESS = 2048
    Symbol NbBytesIDlocs = 8
'-------------------------------------------------------------------------------
'    Include "D:\MyIncFiles\PPS_18F27K40.inc"
'-------------------------------------------------------------------------------
' Declare the UART Baud Rate manually: enable ONE line ONLY.

'    $define _BaudRate 9600

'    $define _BaudRate 19200
    
'    $define _BaudRate 38400
    
'    $define _BaudRate 57600
        
'    $define _BaudRate 115200

'-------------------------------------------------------------------------------
' The Declare UART Baudrate is automatic depending of the Xtal to be syncronized with the bootloader.
    $ifndef _BaudRate
        $if _xtal >= 40
            $ifdef _InternalOSC_
            $define _BaudRate 57600
            $else
            $define _BaudRate 115200
            $endif
        $endif
        $if (_xtal >= 16) And (_xtal < 40)
            $define _BaudRate 57600
        $endif
        $if _xtal < 16
            $define _BaudRate 38400
        $endif
    $endif
'===============================================================================
' Same Baudrate as the Bootloader to check the Software.
'-------------------------------------------------------------------------------
$if _BaudRate = 9600
    Declare Hserial_Baud = 9600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 19200
    Declare Hserial_Baud = 19200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 38400
    Declare Hserial_Baud = 38400
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 57600
    Declare Hserial_Baud = 57600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode  4024
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 115200
    Declare Hserial_Baud = 115200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'===============================================================================
    Symbol CTL_ROM_SIZE = _code
    Symbol CTL_RAM_SIZE = _ram    
'===============================================================================
' Same Config Fuses as Bootloader:
'
$ifdef _InternalOSC_
Config_Start
    FEXTOSC = OFF	          ;Oscillator not enabled
    RSTOSC = HFINTOSC_64MHZ	;HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
    CLKOUTEN = OFF	        ;CLKOUT function is disabled
    CSWEN = On	            ;Writing to NOSC and NDIV is allowed
    FCMEN = OFF	            ;Fail-Safe Clock Monitor disabled
    MCLRE = EXTMCLR	        ;If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
    PWRTE = On	         ;Power up timer enabled
    LPBOREN = On	 ;ULPBOR enabled
    BOREN = On	   ;Brown-out Reset enabled according to SBOREN
    BORV = VBOR_245	;Brown-out Reset Voltage (VBOR) set to 2.45V
    ZCD = OFF	        ;ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
    PPS1WAY = OFF	    ;PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
    STVREN = On	  ;Stack full/underflow will cause Reset
    Debug = OFF	  ;Background debugger disabled
    XINST = OFF	  ;Extended Instruction Set and Indexed Addressing Mode disabled
    WDTCPS = WDTCPS_2	;Divider ratio 1:128
    WDTE = SWDTEN	;WDT enabled/disabled by SWDTEN bit
    WDTCWS = WDTCWS_7	;window always open (100%); software control; keyed access not required
    WDTCCS = SC	;Software Control
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000-00FFFFh) not write-protected
    WRT4 = OFF	;Block 4 (010000-013FFFh) not write-protected
    WRT5 = OFF	;Block 5 (014000-017FFFh) not write-protected
    WRT6 = OFF	;Block 6 (018000-01BFFFh) not write-protected
    WRT7 = OFF	;Block 7 (01C000-01FFFFh) not write-protected
    WRTC = On	;Configuration registers (300000-30000Bh) write-protected
    WRTB = On	;Boot Block (000000-0007FFh) write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    SCANE = On	;Scanner module is available for use, SCANMD bit can control the module
    LVP = On	;Low voltage programming enabled. MCLR/VPP pin function is MCLR. MCLRE configuration bit is ignored
    Cp = OFF	;UserNVM code protection disabled
    CPD = OFF	;DataNVM code protection disabled
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTR4 = OFF	;Block 4 (010000-013FFFh) not protected from table reads executed in other blocks
    EBTR5 = OFF	;Block 5 (014000-017FFFh) not protected from table reads executed in other blocks
    EBTR6 = OFF	;Block 6 (018000-01BFFFh) not protected from table reads executed in other blocks
    EBTR7 = OFF	;Block 7 (01C000-01FFFFh) not protected from table reads executed in other blocks
    EBTRB = On	;Boot Block (000000-0007FFh) protected from table reads executed in other blocks
Config_End
$else
Config_Start
    FEXTOSC = HS	;HS (crystal oscillator) above 8 MHz; PFM set to high power
    RSTOSC = EXTOSC	;EXTOSC operating per FEXTOSC bits (device manufacturing default)
    CLKOUTEN = OFF	;CLKOUT function is disabled
    CSWEN = On	     ;Writing to NOSC and NDIV is allowed
    FCMEN = OFF	    ;Fail-Safe Clock Monitor disabled
    MCLRE = EXTMCLR	;If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
    PWRTE = On	   ;Power up timer enabled
    LPBOREN = On	 ;ULPBOR enabled
    BOREN = On	   ;Brown-out Reset enabled according to SBOREN
    BORV = VBOR_245	;Brown-out Reset Voltage (VBOR) set to 2.45V
    ZCD = OFF	        ;ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
    PPS1WAY = OFF	    ;PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
    STVREN = On	  ;Stack full/underflow will cause Reset
    Debug = OFF	  ;Background debugger disabled
    XINST = OFF	  ;Extended Instruction Set and Indexed Addressing Mode disabled
    WDTCPS = WDTCPS_2	;Divider ratio 1:128
    WDTE = SWDTEN	;WDT enabled/disabled by SWDTEN bit
    WDTCWS = WDTCWS_7	;window always open (100%); software control; keyed access not required
    WDTCCS = SC	;Software Control
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000-00FFFFh) not write-protected
    WRT4 = OFF	;Block 4 (010000-013FFFh) not write-protected
    WRT5 = OFF	;Block 5 (014000-017FFFh) not write-protected
    WRT6 = OFF	;Block 6 (018000-01BFFFh) not write-protected
    WRT7 = OFF	;Block 7 (01C000-01FFFFh) not write-protected
    WRTC = On	;Configuration registers (300000-30000Bh) write-protected
    WRTB = On	;Boot Block (000000-0007FFh) write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    SCANE = On	;Scanner module is available for use, SCANMD bit can control the module
    LVP = On	;Low voltage programming enabled. MCLR/VPP pin function is MCLR. MCLRE configuration bit is ignored
    Cp = OFF	;UserNVM code protection disabled
    CPD = OFF	;DataNVM code protection disabled
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTR4 = OFF	;Block 4 (010000-013FFFh) not protected from table reads executed in other blocks
    EBTR5 = OFF	;Block 5 (014000-017FFFh) not protected from table reads executed in other blocks
    EBTR6 = OFF	;Block 6 (018000-01BFFFh) not protected from table reads executed in other blocks
    EBTR7 = OFF	;Block 7 (01C000-01FFFFh) not protected from table reads executed in other blocks
    EBTRB = On	;Boot Block (000000-0007FFh) protected from table reads executed in other blocks
Config_End
$endif
'###############################################################################
' THE USER SOFTWARE STARTS:
'###############################################################################
'===============================================================================
'    Include "D:\MyIncFiles\SYS_18F27K40_47K40.INC"
'    Include "SYS_18Fxxk22.INC"
'===============================================================================
    Include "D:\MyIncFiles\PPS_18F27_47K40.INC"
'===============================================================================
' New Definitions of the library flags.
    Dim F_TMR2IF As PIR4Bits_TMR2IF     ' TMR2 To PR2 Match Interrupt Flag Bit
    Dim F_TMR2IE As PIE4Bits_TMR2IE     ' TMR2 To PR2 Match Interrupt Enable bi
    Dim F_TMR2ON As T2CONBits_TMR2ON    ' Timer2 On Bit 
    Dim F_IPEN As INTCONBits_IPEN       ' Disable priority levels On interrupts (PIC16CXXX Compatibility mode)
    Dim F_TMR2IP As IPR4Bits_TMR2IP     ' TMR2 to PR2 Match Interrupt Priority bit
'------------------------------------------------------------------------------- 
'ASCII characters  
    Symbol Home = 1
    Symbol Bell = 7
    Symbol BkSp = 8
    Symbol Tab = 9
    Symbol CR = 13
    Symbol LF = $0A     ' Line Feed Char    
    Symbol True = 1     ' Boolean True
    Symbol False = 0    ' Boolean False
    Symbol Null = 0     ' Null character 
    Symbol ACK = 6
    Symbol STX = 2
    Symbol ETX = 3
    Symbol SPACE = $20
    Symbol DEL = 127    
    Symbol ESC = $1B    ' Escape
    Symbol BS = 08      ' Backspace 
'===============================================================================
' All inputs to One PORT only is the best solution for EMC compliance.
'
    $define RisingEdgeMask  %11111111
    $define FallingEdgeMask %11111111

    ' Declaration of the inputs virtual PORT.
    ' Input reading in PORTB        
    $define InputPORT PORTB

    $define InputPin6 PORTA.4
    $define InputPin7 PORTA.5    

'    $define AnselPin7 ANSELABits_ANSA5
    $define AnselPin7 ANSELA.5

    $define VirtualPORT_IdleState %11111111
    ' If there are not others pins used as analog then all PORTA pins could set to digital.
    $define AnselPORT ANSELB
    
    $define _FallingEdgesDetect_
    $define _RisingEdgesDetect_
'-------------------------------------------------------------------------------
' Definition of the Buttons for the input virtual PORT for the AMI18 shield for Amicus18.
    Dim SW0 As DebounceOut.0
    Dim SW1 As DebounceOut.1    
    Dim SW2 As DebounceOut.2    
    Dim SW3 As DebounceOut.3
    Dim SW4 As DebounceOut.4
    Dim SW5 As DebounceOut.5    
    Dim SW6 As DebounceOut.6    
    Dim SW7 As DebounceOut.7          
'-------------------------------------------------------------------------------
' Definition of the LEDS for the output virtual PORT for the AMI18 shield for Amicus18.  #define _CWRITE_BLOCK 64
    Symbol LED0 = LATA.0
    Symbol LED1 = LATA.1
    Symbol LED2 = LATA.2
    Symbol LED3 = LATA.3
    Symbol LED4 = LATC.0
    Symbol LED5 = LATC.1
    Symbol LED6 = LATC.2
    Symbol LED7 = LATC.3
    Symbol LED8 = LATC.4
    Symbol LED9 = LATC.5       
'-------------------------------------------------------------------------------
    Symbol _Config2l = $300002      ' 300002h CONFIG2L BOREN1 BOREN0 LPBOREN    PWRTE MCLRE
    Symbol _Config2h = $300003      ' 300003h CONFIG2H XINST  DEBUG STVREN PPS1WAY ZCD BORV1 BORV0
    Symbol _Config3l = $300004      ' 300004h CONFIG3L  WDTE<1:0> WDTCPS<4:0>
    Symbol _Config3h = $300005      ' 300005h CONFIG3H   WDTCCS<2:0> WDTCWS<2:0>
    Symbol _Config4l = $300006      ' 300006h CONFIG4L WRT7 WRT6 WRT5 WRT4 WRT3 WRT2 WRT1 WRT0  
    Symbol _Config4h = $300007      ' 300007h CONFIG4H   LVP SCANE  WRTD WRTB WRTC 
    Symbol _Config5l = $300008      ' 300008h CONFIG5L       CPD CP
    Symbol _Config6l = $30000A      ' 30000Ah CONFIG6L EBTR7 EBTR6 EBTR5 EBTR4 EBTR3 EBTR2 EBTR1 EBTR0 
    Symbol _Config6h = $30000B      ' 30000Bh CONFIG6H       EBTRB
    Symbol _ID_LOCATION = $200000   ' Write the IDLOCS address. Eight memory locations (200000h-200007h) are designated, see the datasheet
    Symbol ID_ADDRESS = $3FFFFC     ' Read the ID address (2 Words) of PICPartNumber. Location of (DEVID1) in memory map, see the datasheet
'-------------------------------------------------------------------------------
' Define some options of debug or test.
'    $define _TestORG_
'-------------------------------------------------------------------------------
' Declare some variables.
    Dim EEPADDR As NVMADRL.Word     ' NVMADRL and NVMADRH
    Dim Loop As Byte
    Dim Temp1 As Byte
    Dim Temp2 As Byte
    Dim Temp3 As Byte
    Dim Temp4 As Byte        
    Dim DataWord As Word
    Dim EepAddress As Word
    Dim PICPartNumber As Word
    Dim PICRevision As Word
    Dim CounterEep As Byte
    Dim RevASCII As Byte
    Dim Address1 As Word
    Dim Address2 As Word       
    Dim Address3 As Word
    Dim Address4 As Word    
    Dim CodeTestFlags As Byte
    Dim F_LED8_On As CodeTestFlags.0
    Dim F_LED8_Off As CodeTestFlags.1    
    Dim F_LED9_On As CodeTestFlags.2
    Dim F_LED9_Off As CodeTestFlags.3
'===============================================================================
    Include "D:\MyIncFiles\GIEDisable.Inc"
'===============================================================================
' Running with Internal oscillator ONLY.
' CONFIG FUSES:
' FOSC = INTIO67    ;Internal oscillator block, port function on RA6 and RA7
' HFOFST = OFF	    ;HFINTOSC output and ready status are delayed by the oscillator stable status
' Time for HF-INTOSC to Stabilize: 1mS, Time For PLL to Stabilize: 2mS
' Delay for code: 25mS (more or less)
'*******************************************************************************
'* To save code it is possible to disable theses lines of code,                *
'* because the Internal Oscillator is configurated by the Bootloader           *
'* or to change the HFINTOSC frequency                                         *
'* FOSC avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10        *
'*******************************************************************************
' Perform the locking sequence for PPS assignment
' Input     : None
' Output    : None
' Clear PPSLOCKED bit to enable writes
'
$ifndef PPS_EnableWrite
$define PPS_EnableWrite() '
    GIE_Disable()         '
    NVMCON2 = $55         '
    NVMCON2 = $AA         '
    PPSLOCK.0 = 0         '
    GIE_Restore()
$endif

'-------------------------------------------------------------------------------
' Perform the locking sequence for PPS assignment
' Input     : None
' Output    : None
' Set PPSLOCKED bit to disable writes
'
$ifndef PPS_DisableWrite
$define PPS_DisableWrite() '
    GIE_Disable()          '
    NVMCON2 = $55          '
    NVMCON2 = $AA          '
    PPSLOCK.0 = 1          '
    GIE_Restore()
$endif
    
'===============================================================================
' Watchdog configuration.
    WDTCON1 = %00000111         ' Window = 100%, WATCHDOG TIMER CONTROL REGISTER 1
'===============================================================================
    $ifdef _InternalOSC_
    OSCCON1 = %01110000                 ' HFINTOSC is used
'    OSCCON1 = OSCCON2                   ' Copy the Config fuses status
    While OSCSTATBits_HFOR = 0 : Wend   ' Wait until the HFINTOSC is stable
    DelayMS 25                          ' Delay to stabilize the Internal Oscillator. (Mandatory)
    $else
    OSCCON1 = %00100000                 ' EXTOSC + 4xPLL
'    OSCCON1 = OSCCON2                   ' Copy the Config fuses status
    DelayMS 12                          ' Delay to stabilize the Xtal Oscillator.    
    $endif
'===============================================================================
    Include "D:\MyIncFiles\HRsout2StopBits_K40.inc"
'    Include "HRsout2StopBits_K40.inc"
'-------------------------------------------------------------------------------
    ' Debounce 8 Inputs library files with interrupt code.
    Include "D:\MyIncFiles\DebounceInt_K40.inc"
'    Include "DebounceInt.inc"
'-------------------------------------------------------------------------------
    $define ERASE_FLASH_BLOCKSIZE 128        ' For the PIC18F27K40.
    Include "D:\MyIncFiles\ORG_SyncBlock.Inc"
'    Include "ORG_SyncBlock.Inc"
'*******************************************************************************
'* Name    : On_Low_Interrupt                                                  *
'* Purpose : To test the ISR_Low Jump by the Bootloader.                       *
'*         : Clock = FOSC/4 => 64Mhz/4 = 16Mhz                                 *
'*******************************************************************************
    On_Low_Interrupt GoTo ISR_Low
    GoTo OverLowInterrupt
'===============================================================================   
ISR_Low:
    ' Timer is used to Toggle the LED9 every second.
    Context Save
    If PIR0Bits_TMR0IF = 1 Then   ' Check the Timer0 Interrupt flag
        PIR0bits_TMR0IF = 0
        #ifdef WatchDog_Req
        Clrwdt
        #endif
            Toggle LED9             ' LED9 = One second ON, one second OFF.
            If LED9 = 1 Then
                Clear LED8
            Else
                Set LED8
            EndIf
    EndIf
    Context Restore
'===============================================================================
OverLowInterrupt:    
'===============================================================================
MAIN_TEST:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    DelayMS 50
'-------------------------------------------------------------------------------    
    PIR2 = 0
    PIE2 = 0
    IPR2 = 0                    
'
    T4CON = 0
    T6CON = 0
    PWM3CON = 0
    PWM4CON = 0    
' Declare all digital   
    ADCON0 = 0
    ADCON1 = 15
    $ifdef _ADCON2
    ADCON2 = 0    
    $endif
    $ifdef _CM1CON0
    CM1CON0 = 0
    $endif
    $ifdef _CM2CON0
    CM2CON0 = 0
    $endif
    $ifdef _CM2CON1
    CM2CON1 = 0
    $endif
    $ifdef _ANSELA
    ANSELA = 0
    $endif
    $ifdef _ANSELB
    ANSELB = 0
    $endif
    $ifdef _ANSELC
    ANSELC = 0
    $endif
    $ifdef _ANSELD
    ANSELD = 0
    $endif    
' The eeprom is not used for the moment.
    NVMCON1 = 0
' PORTS
    ' TRIS all outputs.
    Output PORTA.0
    Output PORTA.1
    Output PORTA.2
    Output PORTA.3
    Output PORTC.0
    Output PORTC.1    
    Output PORTC.2
    Output PORTC.3
    Output PORTC.4
    Output PORTC.5
    LATA = 0
    LATB = 0
    LATC = 0
'-------------------------------------------------------------------------------
UART_CONFIG:
' Config the UART pins to digital.
    ' The interrupts are disabled.
    PPSLOCK = $55
    PPSLOCK = $AA
    PPSLOCK.0 = 0
    RX1PPS = 23     ' RC7 input
    RC6PPS = 9      ' RC6 output
    PPSLOCK = $55
    PPSLOCK = $AA
    PPSLOCK.0 = 1
'-------------------------------------------------------------------------------
' Config the UART pins to digital with the library.
'    PPS_EnableWrite()
'    iPPSInput(cIn_Fn_RX1DT1,cIn_Pin_RC7)       ' RX1 <= RC7 input 
'    iPPSOutput(cOut_Pin_RC6,cOut_Fn_TX1CK1)    ' RC6 output <= TX1
'    PPS_DisableWrite()
'-------------------------------------------------------------------------------
    ANSELC.7 = 0
    ANSELC.6 = 0
    Input PORTC.7   ' RX1
    Output PORTC.6  ' TX1         
    ODCONC.7 = 0
    SLRCONC = 0
    '
    ' Clean the UART
    Clear RC1STABits_CREN 
    Set RC1STABits_CREN 
    WREG = RC1REG
    $ifdef _2StopBits_
    TX2STABits_TX9 = 1      ' Nine bit mode for 2 stop bits for transmision.
    TX2STABits_TX9D = 1     ' Send the first stop bit. The second stop bit is set by the UART.
    $endif
    DelayMS 50
'*******************************************************************************
' Write some String in the Data eeprom memory.
    '---------------------------------------------------------------------------
E2p_AddressString1 EData "Testing PIC18F27K40 AGV Bootloader V1.0 written in High Side of ROM.",0

E2p_AddressString2 EData "New Universal PIC18 Bootloader by Alberto Freixanet. ",0  ' 53 characters

E2p_AddressString3 EData "A 8 digits Password is needed to download the user code. ",0  ' 57 characters

E2p_AddressString4 EData As Byte 10,10,10,10

'*******************************************************************************
STARTING_TEST:
    ' Declare the debounce timing you want: example 40 => reading the inputs every 40mS.
    ' The value must to be a multiple of 4. Example: 4,8,12,16,20,24,28,32,36,40,44,48,52,56,60, etc...
    '
    DebounceRate(40)
'===============================================================================
' The Configuration of the Timer0 is different.
' Configuration of the Timer0 to toggle a LED every 1.048Sec 
    ' Timer0 is configured As a 16-Bit timer/Counter, 1:256 prescale value: 1.048Sec 
    ' For xtal = 64
    GIE_Disable()               ' Disable all interrupts.
    WPUB = 0                    ' All PORTB pull-ups are disabled.
    INTCONBits_IPEN = 1         ' Enable priority levels on interrupts.
    T0CON0 = %00010000          ' 16 Bits Timer, Postscaler = 0, Timer0 = OFF
    T0CON1 = %01001000          ' FOSC/4, Prescaler = 256 
    IPR0Bits_TMR0IP = 0         ' Timer0 for Low Interrupt
    PIR0Bits_TMR0IF = 0         ' Clear the Timer0 interrupt flag to begin.
    PIE0Bits_TMR0IE = 1         ' Enable the Timer0 interrupts.
    INTCON.6 = 1                ' Enables all low priority interrupts
    TMR0H = 0                   ' Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CON0Bits_T0EN = 1         ' Start the Timer0.
    GIE_Restore()
'===============================================================================
    HRSOut CR,LF,CR,LF
'-------------------------------------------------------------------------------    
    HRSOut "USER CODE STARTS!",CR,LF 
    HRSOut "TESTING THE HRSOut COMMAND!",CR,LF 
'===============================================================================
READ_CONFIG:
' Read config fuses in memory map.
    TBLPTRL = _Config2l & $0000FF
    TBLPTRH = (_Config2l >> 8) & 255
    TBLPTRU = (_Config2l >> 16) & 255
    GIE_Disable()                   ' PIC errata, DS80000713A.
    NVMCON1 = %11000000             ' Access Configuration registers
    Tblrd*+                         ' Read one memory position.  
    Temp3 = TABLAT                  ' 300002h Config2L BOREN1 BOREN0 LPBOREN    PWRTE MCLRE
    Tblrd*+                         ' Read one memory position.  
    Temp2 = TABLAT                  ' 300003h Config2H XINST  Debug STVREN PPS1WAY ZCD BORV1 BORV0
    Tblrd*+                         ' Read one memory position.  
    Temp1 = TABLAT                  ' 300004h CONFIG3L  WDTE<1:0> WDTCPS<4:0>
    NVMCON1 = 0                     ' Disable Access the Configuration registers
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0                     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
'-------------------------------------------------------------------------------
START_CONFIG:
    TBLPTRU = 0
    HRSOut "CONFIG FUSES: ",CR,LF 
'
    HRSOut "Configuration Word 2L: %", Bin8 Temp3,CR,LF
    Temp3 = (Temp3 >> 6) & 3
    If Temp3.5 = 0 Then
        HRSOut "Low-Power Brown-out Reset is enabled!",CR,LF
        If Temp3 = 3 Then
            HRSOut "Brown-out Reset enabled, SBOREN bit is ignored!"    
        ElseIf Temp3 = 2 Then
            HRSOut "Brown-out Reset enabled while running, disabled in Sleep; SBOREN is ignored!"   
        ElseIf Temp3 = 1 Then
            HRSOut "Brown-out Reset enabled according to SBOREN!"
        ElseIf Temp3 = 0 Then
            HRSOut "Brown-out Reset disabled!"
        EndIf
    Else
        HRSOut "Low-Power Brown-out Reset is disabled!"
    EndIf
    HRSOut CR,LF
    If BORCONBits_SBOREN = 1 Then
        HRSOut "Brown-out Reset enabled by SBOREN!" 
    Else
        HRSOut "Brown-out Reset disabled by SBOREN!"     
    EndIf  
    HRSOut CR,LF,CR,LF
'
    HRSOut "Configuration Word 2H: %", Bin8 Temp2,CR,LF
    If Temp2.3 = 1 Then    
        HRSOut "The PPSLOCKED bit can only be set once after an unlocking sequence is executed"
    Else
        HRSOut "The PPSLOCKED bit can be set and cleared as needed"    
    EndIf
    HRSOut CR,LF,CR,LF
'
    HRSOut "Configuration Word 3L: %", Bin8 Temp1,CR,LF
    Temp1 = (Temp1 >> 5) & 3
    If Temp1 = 3 Then
        HRSOut "WDT enabled regardless of Sleep; SEN bit in WDTCON0 is ignored!"    
    ElseIf Temp1 = 2 Then
        HRSOut "WDT enabled While Sleep = 0; Suspended when Sleep = 1; SEN bit in WDTCON0 is ignored!"   
    ElseIf Temp1 = 1 Then
        HRSOut "WDT enabled/disabled by SEN Bit in WDTCON0!",CR,LF
        HRSOut "Be carrefull the WDT could be active!" 
    ElseIf Temp1 = 0 Then
        HRSOut "WDT disabled, SEN Bit in WDTCON0 is ignored!"
    EndIf
    HRSOut CR,LF,CR,LF
'===============================================================================
' Testing moving the code within PIC ROM. The bootloader must detect the offset.
    $ifdef _TestORG_
    GoTo FirstORG11

    Org 6348            ' Standard ORG command. ($18CC)
'    ORG_EBlock(3548)    ' Syncronize the ORIGIN with the ERASE Block. ()
'    ORG_WBlock(3548)    ' Syncronize the ORIGIN with the WRITE Block. ()
    $endif
FirstORG11:    
'===============================================================================
' Check the SWDTEN Bit.
    If Temp1 = 1 Then
        HRSOut "After Reset the "
        If WDTCON0Bits_SWDTEN = 1 Then
            HRSOut "SWDTEN Bit = 1 (error)",CR,LF
        Else
            HRSOut "SWDTEN Bit = 0 (good, by code)",CR,LF
        EndIf
        HRSOut "Now Set the SWDTEN Bit",CR,LF
        WDTCON0Bits_SWDTEN = 1           ' Enable the Watchdog
        If WDTCON0Bits_SWDTEN = 1 Then
            HRSOut "The SWDTEN Bit is set => Watchdog Enabled.",CR,LF
        Else
            HRSOut "The SWDTEN Bit is Cleared => Watchdog Disabled.",CR,LF    
        EndIf
    EndIf
    HRSOut CR,LF
'===============================================================================
    $ifdef _InternalOSC_
    HRSOut "HFINTOSC IS IN USE.",CR,LF 
    $else
    HRSOut "EXTOSC IS IN USE.",CR,LF     
    $endif
    HRSOut "OSCCON1: OSCILLATOR CONTROL REGISTER 1: %", Bin8 OSCCON1,CR,LF          
    HRSOut "OSCCON2: OSCILLATOR CONTROL REGISTER 2: %", Bin8 OSCCON2,CR,LF
    HRSOut "OSCCON3: OSCILLATOR CONTROL REGISTER 3: %", Bin8 OSCCON3,CR,LF
    HRSOut "OSCSTAT: OSCILLATOR STATUS  REGISTER 1: %", Bin8 OSCSTAT,CR,LF
    $ifdef _InternalOSC_
    HRSOut "OSCFRQ:  HFINTOSC FREQUENCY SELECTION REGISTER: %", Bin8 OSCFRQ,CR,LF    
    HRSOut "OSCTUNE: HFINTOSC TUNING REGISTER: %", Bin8 OSCTUNE,CR,LF     
    $endif
    HRSOut CR,LF
'===============================================================================
' Read the PIC ID from Config fuse.
    TBLPTRL = ID_ADDRESS & $0000FF
    TBLPTRH = (ID_ADDRESS >> 8) & 255
    TBLPTRU = (ID_ADDRESS >> 16) & 255
    GIE_Disable()                   ' PIC errata, DS80000713A.
    NVMCON1 = %11000000             ' Access Configuration registers 
    ' Read Revision
    Tblrd*+
    PICRevision.LowByte = TABLAT
    Tblrd*+
    PICRevision.HighByte = TABLAT     
    ' Read Part Number
    Tblrd*+
    PICPartNumber.LowByte = TABLAT
    Tblrd*+
    PICPartNumber.HighByte = TABLAT
    NVMCON1 = 0                     ' Disable Access the Configuration registers
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0                     ' Cleared for security
    '---------------------------------------------------------------------------
    ' Calculate the PIC Part Number & revision
    ' ASCII calculation
    PICRevision.HighByte = (PICRevision.HighByte & %00001111) << 2
    Temp4 = PICRevision.LowByte >> 6
    RevASCII = PICRevision.HighByte | Temp4
    RevASCII = RevASCII + "A"
    ' Number calculation
    PICRevision.LowByte = PICRevision.LowByte & %00111111
'-------------------------------------------------------------------------------
    HRSOut "Device = "
    Select PICPartNumber
        Case 26976
            HRSOut "PIC18F27K40"
        Case 26880
            HRSOut "PIC18F47K40"        
        Case 27008
            HRSOut "PIC18F26K40" 
        Case 26944
            HRSOut "PIC18F45K40"             
        Case 26912
            HRSOut "PIC18F46K40" 
        Case Else
            HRSOut "No correct Part Number"
            PICPartNumber = 0
    EndSelect
    HRSOut CR,LF    
'-------------------------------------------------------------------------------
    HRSOut "PIC Part Number: ", Dec PICPartNumber, "/$", Hex4 PICPartNumber 
    HRSOut "; Revision: ", RevASCII, Dec3 PICRevision.LowByte, "/", RevASCII,"$", Hex2 PICRevision.LowByte,CR,LF
    HRSOut "FOSC: ", Dec _xtal," Mhz;"
    $ifdef _InternalOSC_
    HRSOut " Internal Oscillator ",CR,LF    
    $else
    HRSOut " Xtal Oscillator ",CR,LF    
    $endif
    HRSOut "WRITE BLOCK: ", Dec _block," Bytes",CR,LF 
    HRSOut "ERASE BLOCK: ", Dec ERASE_FLASH_BLOCKSIZE," Bytes",CR,LF
    HRSOut "EEPROM MEMORY: ", Dec _eeprom," Bytes",CR,LF
    HRSOut "ROM MEMORY: ", Dec _code," Bytes",CR,LF     
    HRSOut "RAM MEMORY: ", Dec _ram," Bytes",CR,LF
    HRSOut "BANKA END: ", Dec _bankAend,CR,LF
    HRSOut "RAM BANKS: ", Dec _rambanks,CR,LF
    HRSOut "Number UART: 1 of ", Dec _uart,CR,LF  
    HRSOut "UART Baud Rate: ", Dec _BaudRate,CR,LF
'-------------------------------------------------------------------------------
    HRSOut CR,LF
'===============================================================================   
IDLOCS:
' READING THE IDLOCS...(16 x Words)
    ' Reading the ID Locs written in ASM.
    GoSub Read_IDLOCS_Sub
    HRSOut CR,LF

    ' Writing new 16 ID Locs.
    #ifdef watchdog_req
    Clrwdt
    #endif
    HRSOut "Writing new ID LOCS: (", Dec NbBytesIDlocs, " Bytes)",CR,LF
    HRSOut "$F0 $F1 $F2 $F3 $F4 $F5 $F6 $F7",CR,LF 
    TBLPTRL = _ID_LOCATION & $0000FF
    TBLPTRH = (_ID_LOCATION >> 8) & 255
    TBLPTRU = (_ID_LOCATION >> 16) & 255
    GIE_Disable()                   ' PIC errata, DS80000713A.
    ' Erase the ID LOCS
    NVMCON1 = %11010100             ' Flash acces + Start erase flash + Setup write flash 
    GoSub WriteSequence_Sub
    ' Write to holding register
    NVMCON1 = %11000000             ' Access Configuration registers
    TABLAT = $F0
    For Loop = 0 To NbBytesIDlocs - 1
        Asm
        tblwt*+
        EndAsm
        Inc TABLAT
    Next  
    'Write new ID Locs
    TBLPTRL = _ID_LOCATION & $0000FF
    NVMCON1 = %11000100            ' Setup write flash
    GoSub WriteSequence_Sub
    GIE_Restore()   ' PIC errata, DS80000713A.
    TBLPTRU = 0     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                    ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
'-------------------------------------------------------------------------------
' READING THE IDLOCS...(16 x Words)
    ' Reading the ID Locs written by code.
    GoSub Read_IDLOCS_Sub
'-------------------------------------------------------------------------------
' Erase the ID LOCS for next time
    HRSOut "Erasing the ID LOCS for next Reset.",CR,LF
    TBLPTRL = _ID_LOCATION & $0000FF
    TBLPTRH = (_ID_LOCATION >> 8) & 255
    TBLPTRU = (_ID_LOCATION >> 16) & 255
    GIE_Disable()                   ' PIC errata, DS80000713A.
    NVMCON1 = %11010100             ' Flash acces + Start erase flash + Setup write flash 
    GoSub WriteSequence_Sub    
    GIE_Restore()                   ' PIC errata, DS80000713A.
    TBLPTRU = 0    
'===============================================================================
STRING_EEP:
    ' Send a String to the terminal read from the data eeprom memory.
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "START EEPROM Strings:",CR,LF
    EepAddress = E2p_AddressString1
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString2
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString3
    GoSub ReadEEPString_Sub
    HRSOut "END EEPROM Strings:",CR,LF,CR,LF
'-------------------------------------------------------------------------------
STRING_ROM:
    HRSOut "START Checking Cread command.",CR,LF
    EepAddress = CdataAddress
    GoSub ReadROMString_Sub
    HRSOut "END Checking CRead command.",CR,LF,CR,LF
'-------------------------------------------------------------------------------
    ' Send a String read from the ROM memory.
    ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
    HRSOut "Debounce 8 Buttons by interrupt "
    HRSOut "and detect the edges change.",CR,LF   
    HRSOut "Timing value: ", Dec DebounceTimingTrigg, " mS",CR,LF
    HRSOut "Debounce value: Timing X 4 = ", Dec DebounceTimingTrigg * 4, " mS",CR,LF    
    HRSOut "Using 8 push buttons and 10 LEDs to do the tests.",CR,LF 
    HRSOut CR,LF

    $ifdef _TestORG_
    HRSOut "Hashed code activated!",CR,LF 
    HRSOut CR,LF
    $endif
'-------------------------------------------------------------------------------
E2P:
    HRSOut "Checking Eread/Ewrite commands.",CR,LF 
    Address1 = E2p_AddressString4
    Address2 = E2p_AddressString4 + 1
    Address3 = E2p_AddressString4 + 2
    Address4 = E2p_AddressString4 + 3   
    HRSOut "Ewrite numbers: 50,125,100,200",CR,LF 
    EWrite  Address1,[50,125,100,200]
    Temp1 = ERead Address1
    Temp2 = ERead Address2
    Temp3 = ERead Address3
    Temp4 = ERead Address4    
    HRSOut "Eread  numbers: ", Dec Temp1,",", Dec Temp2,",", Dec Temp3,",", Dec Temp4,CR,LF 
    HRSOut CR,LF
'-------------------------------------------------------------------------------
    HRSOut "TESTING THE BUTTONS AND LEDS!",CR,LF 
    HRSOut "LED8 is Set when all LEDs are ON!",CR,LF 
    HRSOut "LED9 is Set when all LEDs are OFF!",CR,LF
    HRSOut CR,LF        
'===============================================================================
' Testing moving the code within PIC ROM. The bootloader must detect the offset.
    $ifdef _TestORG_
    GoTo FirstORG22

    Org 12004                ' Standard ORG command. ($2EE4)
'    ORG_EBlock(11004)    ' Syncronize the ORIGIN with the ERASE Block. ()
'    ORG_WBlock(11004)    ' Syncronize the ORIGIN with the WRITE Block. ()
    $endif
FirstORG22:    
'===============================================================================      
    CodeTestFlags = 0               ' All flags cleared to start.
    GoSub LEDsOn_or_LEDsOff_Sub     ' Print the initial LEDs state.
'-------------------------------------------------------------------------------
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        ' Check if there is a falling edge.
        If FallingEdgeDetectedBits.0 = 1 Then
            FallingEdgeDetectedBits.0 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW0"
            If LED0 = 0 Then
                Set LED0
                HRSOut ", Set"
            Else
                Clear LED0
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED0",CR,LF          
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf

        If FallingEdgeDetectedBits.1 = 1 Then
            FallingEdgeDetectedBits.1 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW1"
            If LED1 = 0 Then
                Set LED1
                HRSOut ", Set"
            Else
                Clear LED1
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED1",CR,LF  
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf            

        If FallingEdgeDetectedBits.2 = 1 Then
            FallingEdgeDetectedBits.2 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW2"
            If LED2 = 0 Then
                Set LED2
                HRSOut ", Set"
            Else
                Clear LED2
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED2",CR,LF  
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf

        If FallingEdgeDetectedBits.3 = 1 Then
            FallingEdgeDetectedBits.3 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW3"
            If LED3 = 0 Then
                Set LED3
                HRSOut ", Set"
            Else
                Clear LED3
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED3",CR,LF     
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf            

        If FallingEdgeDetectedBits.4 = 1 Then
            FallingEdgeDetectedBits.4 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW4"
            If LED4 = 0 Then
                Set LED4
                HRSOut ", Set"
            Else
                Clear LED4
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED4",CR,LF    
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf

        If FallingEdgeDetectedBits.5 = 1 Then
            FallingEdgeDetectedBits.5 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW5"
            If LED5 = 0 Then
                Set LED5
                HRSOut ", Set"
            Else
                Clear LED5
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED5",CR,LF 
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf            

        If FallingEdgeDetectedBits.6 = 1 Then
            FallingEdgeDetectedBits.6 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW6"
            If LED6 = 0 Then
                Set LED6
                HRSOut ", Set"
            Else
                Clear LED6
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED6",CR,LF 
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf

        If FallingEdgeDetectedBits.7 = 1 Then
            FallingEdgeDetectedBits.7 = 0       ' Clear the bit when it is already read.
            HRSOut "Falling Edge Detect: SW7"
            If LED7 = 0 Then
                Set LED7
                HRSOut ", Set"
            Else
                Clear LED7
                HRSOut ", Clear"            
            EndIf
            HRSOut " LED7",CR,LF 
            GoSub LEDsOn_or_LEDsOff_Sub         ' Write the status of all LEDs.
        EndIf   
        '=======================================================================
        ' Testing moving the ROM code within PIC ROM. The bootloader must detect the offset.
        $ifdef _TestORG_

        GoTo FirstORG33
        Org 24556                   ' Standard ORG command. ($5FEC)

'        ORG_EBlock(23556)       ' Syncronize the ORIGIN with the ERASE Block.
'        ORG_WBlock(23556)       ' Syncronize the ORIGIN with the WRITE Block.  
        $endif
FirstORG33:    
        '=======================================================================
        ' Check if there is a rising edge.
        If RisingEdgeDetectedBits.0 = 1 Then
            RisingEdgeDetectedBits.0 = 0        ' Clear the bit when it is already read.
            HRSOut "SW0 depressed",CR,LF,CR,LF
        EndIf

        If RisingEdgeDetectedBits.1 = 1 Then
            RisingEdgeDetectedBits.1 = 0        ' Clear the bit when it is already read.
            HRSOut "SW1 depressed",CR,LF,CR,LF
        EndIf            

        If RisingEdgeDetectedBits.2 = 1 Then
            RisingEdgeDetectedBits.2 = 0        ' Clear the bit when it is already read.
            HRSOut "SW2 depressed",CR,LF,CR,LF
        EndIf

        If RisingEdgeDetectedBits.3 = 1 Then
            RisingEdgeDetectedBits.3 = 0        ' Clear the bit when it is already read.
            HRSOut "SW3 depressed",CR,LF,CR,LF
        EndIf            

        If RisingEdgeDetectedBits.4 = 1 Then
            RisingEdgeDetectedBits.4 = 0        ' Clear the bit when it is already read.
            HRSOut "SW4 depressed",CR,LF,CR,LF
        EndIf
        
        If RisingEdgeDetectedBits.5 = 1 Then
            RisingEdgeDetectedBits.5 = 0        ' Clear the bit when it is already read.
            HRSOut "SW5 depressed",CR,LF,CR,LF
        EndIf
        
        If RisingEdgeDetectedBits.6 = 1 Then
            RisingEdgeDetectedBits.6 = 0        ' Clear the bit when it is already read.
            HRSOut "SW6 depressed",CR,LF,CR,LF
        EndIf                

        If RisingEdgeDetectedBits.7 = 1 Then
            RisingEdgeDetectedBits.7 = 0        ' Clear the bit when it is already read.
            HRSOut "SW7 depressed",CR,LF,CR,LF
        EndIf
'        DelayMS 10
    '---------------------------------------------------------------------------
    Wend
'===============================================================================
'SUBROUTINES:
'===============================================================================
    $ifdef _TestORG_
    Org 32356                   ' Standard ORG command. ($5FEC)
    
'    ORG_EBlock(29356)       ' Syncronize the ORIGIN with the ERASE Block. (29350)
'    ORG_WBlock(29356)       ' Syncronize the ORIGIN with the WRITE Block.  
    $endif
'===============================================================================
LEDsOn_or_LEDsOff_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    Temp1 = LATA & 15
    Temp2 = LATC & 15
    ' Detect if all LEDs are ON.
    If Temp1 = 15 And Temp2 = 15 Then
        Set LED8
        HRSOut "LED8 On: All LEDs ON",CR,LF
        F_LED8_On = 1
    Else
        Clear LED8        
        If F_LED8_On = 1 Then
            F_LED8_On = 0
            HRSOut "LED8 Off",CR,LF
        EndIf
    EndIf

    ' Detect if all LEDs are OFF.
    If Temp1 = 0 And Temp2 = 0 Then
        Set LED9
        HRSOut "LED9 On: All LEDs OFF",CR,LF
        F_LED9_Off = 1
    Else
        Clear LED9        
        If F_LED9_Off = 1 Then
            F_LED9_Off = 0
            HRSOut "LED9 Off",CR,LF
        EndIf
    EndIf
    Return
'===============================================================================
' Read a string written in the eeprom memory.
ReadEEPString_Sub:
    Loop = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = ERead EepAddress + Loop
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            Inc CounterEep
            If CounterEep > 10 Then     ' Too much addresses erased.
                HRSOut "This String is erased!",CR,LF
                Return     
            EndIf
        Else
            HRSOut Temp1
        EndIf
        Inc Loop
    Wend 
'===============================================================================
' Read a string written in the ROM memory.
ReadROMString_Sub:
    Loop = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = CRead EepAddress + Loop
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            HRSOut "This String is erased!",CR,LF
            Return     
        Else
            HRSOut Temp1
        EndIf
        Inc Loop
    Wend     
'===============================================================================
Read_IDLOCS_Sub:
' READING THE IDLOCS...(8 x Words)
    HRSOut "Reading the ID LOCS: (", Dec NbBytesIDlocs, " Bytes)",CR,LF
    TBLPTRL = _ID_LOCATION & 255
    TBLPTRH = (_ID_LOCATION >> 8) & 255
    TBLPTRU = (_ID_LOCATION >> 16) & 255
    GIE_Disable()                       ' PIC errata, DS80000713A.
    NVMCON1 = %11000000                 ' Access Configuration registers
    For Loop = 0 To NbBytesIDlocs - 1   ' Start the loop
        Tblrd*+                         ' Read one memory position.
        HRSOut "$"
        HRSOut Hex2 TABLAT
        'HRSOut Dec TABLAT
        HRSOut " "
'        If Loop = 7 Then
'            HRSOut CR,LF
'        EndIf
    Next  
    NVMCON1 = 0     ' Disable read the Configuration registers
    GIE_Restore()   ' PIC errata, DS80000713A.
    TBLPTRU = 0     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                    ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    HRSOut CR,LF  
    Return
'===============================================================================
WriteSequence_Sub:
    NVMCON2 = $55                    ' EECON2 <- $55
    NVMCON2 = $AA                    ' EECON2 <- $AA
    Set NVMCON1Bits_WR               ' Set WR Bit For write
    Nop
    While NVMCON1Bits_WR = 1 : Wend  ' Waiting to complete.
    NVMCON1 = 0                      ' Disable write
    Return
'===============================================================================
CdataAddress:   CData "HELLO WORLD",0
'===============================================================================
    End
'===============================================================================
' Write some static data to the ID Locations.
' Problem writing ID LOCS in static mode.
Asm
__IDLOCS   __IDLOC0,0x77
__IDLOCS   __IDLOC1,0x76
__IDLOCS   __IDLOC2,0x75
__IDLOCS   __IDLOC3,0x74
__IDLOCS   __IDLOC4,0x73
__IDLOCS   __IDLOC5,0x72
__IDLOCS   __IDLOC6,0x71
__IDLOCS   __IDLOC7,0x70
EndAsm

'Asm
'__IDLOCS   0X200000,0xFF
'__IDLOCS   0X200001,0xFF
'__IDLOCS   0X200002,0xFF
'__IDLOCS   0X200003,0xFF
'__IDLOCS   0X200004,0xFF
'__IDLOCS   0X200005,0xFF
'__IDLOCS   0X200006,0xFF
'__IDLOCS   0X200007,0xFF
'EndAsm
'===============================================================================
'===============================================================================


